import java.awt.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;


public class FileObject extends TransformGroup {
  private Cylinder cylinder;
  private File file;
  private Transform3D textRotateT3D;
  private TransformGroup textRotateTrans;
  private Transform3D originalPositionT3D;
  private TransformGroup originalPositionTrans;
  private Transform3D moveT3D;
  private TransformGroup moveTrans;
  private Font3D font3d;
  private Text3D text;
  private Shape3D textShape;
  private Brahman brahman;
  private float radius=1, height, health, d;
  private Color3f writableSpecularColor= new Color3f(0,0,1);
  private Color3f writableDiffuseColor= new Color3f(0,0,.72f);
  private Color3f writableEmissiveColor= new Color3f(0,0,.2f);
  private Color3f notWritableSpecularColor= new Color3f(1,0,0);
  private Color3f notWritableDiffuseColor= new Color3f(.72f,0,0);
  private Color3f notWritableEmissiveColor= new Color3f(.2f,0,0);


  public FileObject(File file,Vector3f position, Brahman brahman) {
    this.file = file;
    this.brahman= brahman;
    setPickable(true);
    //setCollidable(true);
    setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    createShape(position);
	 health=height;
  }
    
  public FileObject(String filePath,Vector3f position, Brahman brahman) {
    file = new File(filePath);
    this.brahman= brahman;
    setPickable(true);
    //setCollidable(true);
    setCapability(TransformGroup.ENABLE_PICK_REPORTING);
    createShape(position);
  }

  void rotText(Kshatriya k) {
    Vector3f myPV=new Vector3f();
    Vector3f kPV;
    float dx,dz;       
    moveT3D.get(myPV);
    kPV = k.getPositionVector();
    dx = kPV.x -myPV.x ;
    dz =  kPV.z -myPV.z;
    originalPositionT3D.get(myPV);
    dx-=myPV.x;
    dz-=myPV.z;

    textRotateT3D.rotY(Math.atan2(dx,dz));
    textRotateTrans.setTransform(textRotateT3D);
  }
    
  float calculateHeight() {
    float minHeight=0.1f,maxHeight=3, height;
    int size = (int)file.length();
    if(size > 1) {
      height= (float)Math.log(size)/20*maxHeight;
      if(height<minHeight)
        return minHeight;
      else if(height>maxHeight)
        return maxHeight;
      else
        return height;
    } else {
      return minHeight;
    }
  }

  public boolean stabIntersect(Vector3f p) {
    float dx,dz;
    Vector3f v = new Vector3f();
    Vector3f w = new Vector3f();
    moveT3D.get(v);
    originalPositionT3D.get(w);
    dx=w.x-v.x-p.x;
    dz=w.z-v.x-p.z;
    if(Math.sqrt(dx*dx+dz*dz)<=radius && p.y<height) return true;
    else return false;
  }

  public void rm( float d ) {
    if(file.canWrite()) {
      health-=d;
      if( health<=0 ) {
        Vector3f v = new Vector3f(0,10,0);
        moveT3D.setTranslation(v);
        moveTrans.setTransform(moveT3D);
        if( !brahman.getSafeMode() ) {
          System.out.println("Removing " + file.getName());
          file.delete();
        }
      }	
    }
  }
  
  TransformGroup createText(float height) {
    Transform3D t3d = new Transform3D();
    t3d.setTranslation(new Vector3f(0f,height/2+0.2f,0f));
    TransformGroup textMoveTrans = new TransformGroup(t3d);

    textRotateT3D = new Transform3D();
    textRotateTrans = new TransformGroup();
    textRotateTrans.setCapability(ALLOW_TRANSFORM_WRITE);
    textRotateTrans.setTransform(textRotateT3D);
    textMoveTrans.addChild(textRotateTrans);
    //Billboard b= new Billboard( textRotateTrans, Billboard.ROTATE_ABOUT_AXIS, new Vector3f( 0,1,0 ) );
    t3d = new Transform3D();
    t3d.setScale(.25f);
    TransformGroup objScale = new TransformGroup(t3d);
    textRotateTrans.addChild(objScale);
	
    String textString = file.getName();
    //Gyulaikod!
    //Jag behvde ett appearance snabbt, frlt Daniel :) /Henrik
    Material mtrl= new Material();
    mtrl.setDiffuseColor( new Color3f( .5f,.7f,.9f));
    mtrl.setSpecularColor( new Color3f( .05f,.07f,.09f));
    Appearance app= new Appearance();
    app.setMaterial( mtrl);
    //Slut p Gyulaikod

    font3d = new Font3D( new Font("Serif", Font.PLAIN, 1),new FontExtrusion());
    text = new Text3D( font3d, textString);
    text.setAlignment( 0 );
    textShape = new Shape3D( text);
    textShape.setAppearance( app );

    objScale.addChild(textShape);
    return textMoveTrans;
  }

  void createShape(Vector3f positionVector) {
    height=calculateHeight();

    originalPositionT3D = new Transform3D();
    originalPositionT3D.setTranslation(positionVector);
    originalPositionTrans = new TransformGroup(originalPositionT3D);
    addChild(originalPositionTrans);

    moveT3D = new Transform3D();
    moveT3D.setTranslation(new Vector3f(0f,height/2f,0f));
    moveTrans = new TransformGroup(moveT3D);
    moveTrans.setCapability(ALLOW_TRANSFORM_WRITE);
    originalPositionTrans.addChild(moveTrans);

    Appearance App= new Appearance();
    Material material= new Material();
    material.setShininess(128);
    if(file.isHidden()) {
      TransparencyAttributes ta = new TransparencyAttributes(TransparencyAttributes.BLENDED,.4f);
      App.setTransparencyAttributes(ta);
    }
    if(file.canWrite()) {
      material.setDiffuseColor(writableDiffuseColor);
      material.setSpecularColor(writableSpecularColor);
      material.setEmissiveColor(writableEmissiveColor);
    } else {
      material.setDiffuseColor(notWritableDiffuseColor);
      material.setSpecularColor(notWritableSpecularColor);
      material.setEmissiveColor(notWritableEmissiveColor);
    }
    App.setMaterial( material );
    cylinder = new Cylinder(radius,height,App);
    moveTrans.addChild(cylinder);
    moveTrans.addChild(createText(height));
  }
}
